//Modification of Glenn Josefiak's Fourier Transform program
// Carl Weiman, 11/7/02

package NextSqPT;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.text.*;

class SquarePulseTransform extends JFrame{
	JTextField indexField = new JTextField("5", 10);
	JTextField dutyCycleField = new JTextField(".5",10);
	JTextField realOutputField = new JTextField(10);
	JTextField imagOutputField = new JTextField(10);
	JButton computeButton = new JButton("Compute");
	JTextArea tableArea = new JTextArea(20, 50);
	
	// Constructor
	public SquarePulseTransform(){
		super("Square Pulse Transform Coefficients, Take 2");

		setSize(650, 600);
		//setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		realOutputField.setEditable(false);
		imagOutputField.setEditable(false);
		tableArea.setEditable(false);

		Container c = getContentPane();
		
		c.setLayout(new FlowLayout());
		c.add(new JLabel("coeff index k"));
		c.add(indexField);
		c.add(new JLabel("duty cycle"));
		c.add(dutyCycleField);
		c.add(new JLabel("coeff real part"));
		c.add(realOutputField);
		c.add(new JLabel("coeff imag part"));
		c.add(imagOutputField);
		c.add(computeButton);
		c.add(tableArea);
		
		KeyListener localKeyListener = new KeyListener(){
			public void keyPressed(KeyEvent e){
			}		
			public void keyReleased(KeyEvent e){
			}		
			public void keyTyped(KeyEvent e){
				argumentChanged();
			}		
		};
		indexField.addKeyListener(localKeyListener);
		dutyCycleField.addKeyListener(localKeyListener);
		
		ActionListener localButtonListener = new ActionListener(){
			public void actionPerformed(ActionEvent e){
				computeButtonPressed();
			}
		};
		computeButton.addActionListener(localButtonListener);
		
	}

	private double computeCoeffReal(int k, double dutyCycle){
		double two_pi_k = 2.0 * Math.PI * k;
		
		if (k == 0)
//			return dutyCycle;
			return 1.0;

		else
//			return (1/two_pi_k) * Math.sin(two_pi_k * dutyCycle);
			return (1/two_pi_k) * Math.sin(two_pi_k * dutyCycle)/dutyCycle;

	}

	private double computeCoeffImag(int k, double dutyCycle){
		double two_pi_k = 2.0 * Math.PI * k;
	
		if (k == 0)
			return 0;
		else
//			return (1/two_pi_k) * (Math.cos(two_pi_k * dutyCycle) - 1.0);
			return (1/two_pi_k) * (Math.cos(two_pi_k * dutyCycle) - 1.0)/dutyCycle;

	}
	
	// round a number to 5 decimal places for display
	private String formatNumber(double x){
		DecimalFormat df = new DecimalFormat();
		df.setMinimumFractionDigits(0);
		df.setMaximumFractionDigits(5);
		
//		return df.format(x + 5e-11);
		return df.format(x);
	}
	
	private void argumentChanged(){
		realOutputField.setText("");
		imagOutputField.setText("");
		tableArea.setText("");
	}

	private void computeButtonPressed(){
		double dutyCycle;
		int k;
		
		try{
			dutyCycle = Double.parseDouble(dutyCycleField.getText());
			k = Integer.parseInt(indexField.getText());
			
			if (dutyCycle <0 || dutyCycle > 1){
				throw new Exception();
			}
			
		}catch (Exception e){
			JOptionPane.showMessageDialog(this, "bad input");
			return;
		}
		
		realOutputField.setText(
				formatNumber(computeCoeffReal(k, dutyCycle))
			);
		imagOutputField.setText(
				formatNumber(computeCoeffImag(k, dutyCycle))
			);
			
		tableArea.append("alpha\tk\tRe(c(k))\tIm(c(k))\tMagnitude\tPhase Angle\tPower Spectrum\n");
		tableArea.append("-----\t---\t--------\t--------\t------------\t------------\t------------\n");
		
		double mag0 = computeCoeffReal(0, dutyCycle);
		
		// Compute and display first 8 coefficients (+ & -)
		for (k=0; k<=8; k++){
			double coefRe;
			double coefIm;
			double mag;
			String line;
			String graph = "";
			
			// Positive k
			coefRe = computeCoeffReal(k, dutyCycle);
			coefIm = computeCoeffImag(k, dutyCycle);
			mag = magnitude(coefRe, coefIm);	

			// Bar graph of power spectrum (scale ratio to 0(th) coefficient by 10)
			for (int m = 0; m < 10*mag/mag0; m++){
				graph += "*";		
			}

			line = dutyCycle + "\t" + 
			       k + "\t" + 
			       formatNumber(coefRe) + "\t" + 
			       formatNumber(coefIm) + "\t" + formatNumber(mag)+
			       "\t" + formatNumber(Math.atan2(coefIm,coefRe)*180.0/Math.PI) + "\t" + graph +"\n";
			tableArea.append(line);	
			
			// Negative k
			coefRe = computeCoeffReal(-k, dutyCycle);
			coefIm = computeCoeffImag(-k, dutyCycle);

			line = dutyCycle + "\t" + 
			       -k + "\t" + 
			       formatNumber(coefRe) + "\t" + 
			       formatNumber(coefIm) + "\t" + formatNumber(mag)+ "\n\n";
			tableArea.append(line);	

		}		
	}

	double magnitude(double x, double y){
		return Math.sqrt(x*x + y*y);	
	}
	
	public static void main(String args[]){
		SquarePulseTransform transformFrame = new SquarePulseTransform();
		transformFrame.show();
	}
}