import javax.swing.*;

public class DFT_Example extends JFrame{
	Complex inputSignal[];
	Complex transform[];
	Complex inverse[];
	int N;
	
	JTextArea textArea = new JTextArea(80, 60);

	private static String THETA = "\u0398"; // Unicode theta

	//Constructor
	public DFT_Example(){
		setTitle("DFT and inverses");
		getContentPane().add(new JScrollPane(textArea));
		setSize(500, 600);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public void go2(){
		// Inject new Coefficients
		N= 2;
		transform = new Complex[N];
		transform[0] = new Complex(0.5f, 0.0f);
		transform[1] = new Complex(-.5f, 0.0f);
		inverse = DFT.computeInverse(transform);

		textArea.append("  Transform:\n");
		textArea.append("  --------------\n");
		append(textArea, "transform", transform);

		textArea.append("  Inverse:\n");
		textArea.append("  ------------\n");
		append(textArea, "inverse", inverse);


	}


	// Major activity of the class
	public void go(){
		
		N = 8;
		// Define input signal
		inputSignal = new Complex[N];
		if ( N ==1 ){
			textArea.append("(Note: For N = 1, I'm arbitrarily setting the function to = .5 ! )\n\n");
			inputSignal[0] = new Complex(0.5f, 0f);
		}
		else
			for (int i = 0; i < inputSignal.length; i++) 
				inputSignal[i] = new Complex((float)i/(float)inputSignal.length, 0);
				
				/* Bandpass filter FT
				if ( i < inputSignal.length/2 )
					inputSignal[i] = new Complex(0, 0);
				else
					inputSignal[i] = new Complex(1, 0);
				*/
				
		// DFT Transform and inverse
		transform = DFT.computeTransform(inputSignal);
		inverse = DFT.computeInverse(transform);

		// Display original, transform, and inverse
		textArea.append("  Original Signal, N = " + N + "\n");
		textArea.append("  ------------------------------\n");
		append(textArea, "inputSignal", inputSignal);

		textArea.append("  Transform:\n");
		textArea.append("  --------------\n");
		append(textArea, "transform", transform);

		textArea.append("  Inverse:\n");
		textArea.append("  ------------\n");
		append(textArea, "inverse", inverse);

	}
	
	// Display utility method
	private void append(JTextArea jta, String name, Complex[] array){
		for (int i = 0; i < array.length; i++){
			jta.append(" " + name + "[" + i + "] = " + formatComplex(array[i]) + "\t");
			jta.append("r = " + formatReal(array[i].getMagnitude()) + "\t");
			jta.append(THETA + " = " + formatReal(array[i].getRadians()) +  "\t" +
				"deg" + " = " + formatReal(array[i].getDegrees()) +"\n");
		}
		jta.append("\n");
	}
	
	// Formatting methods for output
	public String formatReal(float x){
		x = Math.round(x*1000.0)/1000f;
		return new Float(x).toString();
	}	
	
	public String formatComplex(Complex x){
		float r = x.getReal();
		float i = x.getImag();
		
		// round for display
		r = Math.round(r*1000.0)/1000f;
		i = Math.round(i*1000.0)/1000f;
	
		// convert to string
		return new Float(r).toString() +
			(i>=0?" + ":" - ") +
			Math.abs(i) +
			"j";
	}

	// main
	public static void main(String[] args){
		DFT_Example x = new DFT_Example();
		x.show();
		x.go2();
	}
}