package gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import gui.*;
import audio.*;

/********************************************************/
/**
 * <b>
 * A sound recorder/player application.
 * <p>
 * Fairfield University<br>
 * CR310/SW511:
 * <i>Digital Audio with Java</i>
 * </b>
 */
public class JCapture extends JFrame{
    AudioPlayer player = new AudioPlayer();
    File currentDirectory;
	
    JToolBar     tbrButtonBar = new JToolBar();
	ActionButton btnCapture;
	ActionButton btnStop;
	ActionButton btnPlay;
	ActionButton btnSave;
	ActionButton btnOpen;
	JComboBox    cmbZoomSelect;

	JMenuBar  mbrMenuBar = new JMenuBar();
	JMenu     mnuFile;
	JMenuItem mnuFileOpen;
	JMenuItem mnuFileSave;
	JMenu     mnuProcess;
	JMenuItem mnuProcessDFT;
	JMenu     mnuHelp;
	JMenuItem mnuHelpAbout;
    
    JPanel     pnlStatusBar = new JPanel();
    JTextField txtState = new JTextField(5);
    JTextField txtAudioFormat = new JTextField(35);

	ScopePanel scopeDisplay = new ScopePanel();

	/********************************************************/
	/**
	 * Initialize the application without displaying the GUI.
	 */
    public JCapture() {
		super("Audio Capture Tool");

		// Set up the toolbar

		tbrButtonBar.setFloatable(false);

        btnOpen = new ActionButton(new ImageIcon("images/icon_open.gif")){
            public void actionPerformed(ActionEvent e){
           		AudioFileChooser afc = new AudioFileChooser(currentDirectory);
           		afc.setDialogType(AudioFileChooser.OPEN_DIALOG);
           		afc.show();
           		if (afc.getSelectedFile() != null){
                	player.openAudioFile(afc.getSelectedFile());
                	currentDirectory = afc.getCurrentDirectory();
                }
            }
        };
        tbrButtonBar.add(btnOpen);

        btnSave = new ActionButton(new ImageIcon("images/icon_save.gif")){
            public void actionPerformed(ActionEvent e){
				if (!player.hasAudio()){
					messageBox("Nothing to save.");
					return;
				}
           		AudioFileChooser afc = new AudioFileChooser(currentDirectory);
           		afc.setDialogType(AudioFileChooser.SAVE_DIALOG);
           		afc.setSourceFormat(player.getFormat());
           		afc.show();
           		if (afc.getSelectedFile() != null){
                	player.saveAudioFile(afc.getSelectedFile(), 
                	                     afc.getSelectedEncoding());
                	currentDirectory = afc.getCurrentDirectory();
                }
            }
        };
        tbrButtonBar.add(btnSave);

		btnCapture = new ActionButton(new ImageIcon("images/icon_record.gif")){
            public void actionPerformed(ActionEvent e){
                player.captureAudio();
            }
        };
        tbrButtonBar.add(btnCapture);
        
        btnStop = new ActionButton(new ImageIcon("images/icon_stop.gif")){
            public void actionPerformed(ActionEvent e){
                player.stop();
            }
        };
        tbrButtonBar.add(btnStop);
        
        btnPlay = new ActionButton(new ImageIcon("images/icon_play.gif")){
            public void actionPerformed(ActionEvent e){
               	player.playAudio();
            }
        };
        tbrButtonBar.add(btnPlay);
        
        String[] zoomOptions = {"1:1", "1:2", "1:4", "1:8", "1:16", "1:32", "1:64"};
        cmbZoomSelect = new JComboBox(zoomOptions);
        cmbZoomSelect.setMaximumSize(new Dimension(60, 25));
        tbrButtonBar.add(cmbZoomSelect);
		cmbZoomSelect.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				scopeDisplay.setZoom(cmbZoomSelect.getSelectedIndex());			
			}
		});

		// set up the menu

		mnuFile = new JMenu("File");
		mnuFileOpen = new JMenuItem("Open");
		mnuFileOpen.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				btnOpen.actionPerformed(e);
			}
		});
		mnuFile.add(mnuFileOpen);
		mnuFileSave = new JMenuItem("Save");
		mnuFileSave.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				btnSave.actionPerformed(e);
			}
		});
		mnuFile.add(mnuFileSave);
		mbrMenuBar.add(mnuFile);

		mnuProcess = new JMenu("Process");
		mnuProcessDFT = new JMenuItem("DFT");
		mnuProcessDFT.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				messageBox("Not implemented yet");
			}
		});
		mnuProcess.add(mnuProcessDFT);
		mbrMenuBar.add(mnuProcess);

		mnuHelp = new JMenu("Help");
		mnuHelpAbout = new JMenuItem("About");
		mnuHelpAbout.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				messageBox(
					"JCapture version 8\n" +
					"Fairfield University CR310/SW511\n\n" +
					"JRE version " + 
					System.getProperty("java.version") + "\n" +
					System.getProperty("os.name") + " version " 
					+ System.getProperty("os.version")
				);			
			}
		});
		mnuHelp.add(mnuHelpAbout);
		mbrMenuBar.add(mnuHelp);

		// set up the status bar
		
		txtState.setEditable(false);
		txtAudioFormat.setEditable(false);
		pnlStatusBar.add(txtState);
		pnlStatusBar.add(txtAudioFormat);

		// set up listeners		

		addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e)
			{
				txtState.setLocation(0, 0);
				txtState.setSize((int)(pnlStatusBar.getWidth()*.15), pnlStatusBar.getHeight());
				txtAudioFormat.setLocation((int)(pnlStatusBar.getWidth()*.15), 0);
				txtAudioFormat.setSize((int)(pnlStatusBar.getWidth()*.85), pnlStatusBar.getHeight());
			}			
		});
         
		player.addAudioPlayerListener(new AudioPlayerListener(){
			public void audioStateChanged(AudioPlayer.State s){
				txtState.setText(s.toString());
				updateUIStrings();
				
				if (s == AudioPlayer.State.IDLE){
					btnPlay.setEnabled(player.hasAudio());
					btnCapture.setEnabled(true);
					btnStop.setEnabled(false);
					btnSave.setEnabled(player.hasAudio());
					mnuFileSave.setEnabled(player.hasAudio());
					btnOpen.setEnabled(true);
					mnuFileOpen.setEnabled(true);
					cmbZoomSelect.setEnabled(player.hasAudio());
					
			       	scopeDisplay.setDataSet(player.getGraphData());
					
				}else if (s == AudioPlayer.State.PLAYING ||
				          s == AudioPlayer.State.RECORDING){
					btnCapture.setEnabled(false);
					btnStop.setEnabled(true);
					btnSave.setEnabled(false);
					mnuFileSave.setEnabled(false);
					btnOpen.setEnabled(false);
					mnuFileOpen.setEnabled(false);
					cmbZoomSelect.setEnabled(false);
				}else if (s == AudioPlayer.State.STOPPING){
					btnStop.setEnabled(false);
					btnPlay.setEnabled(false);
					btnCapture.setEnabled(false);
				}else if (s == AudioPlayer.State.OPENING ||
					      s == AudioPlayer.State.SAVING){
					btnStop.setEnabled(false);
					btnPlay.setEnabled(false);
					btnCapture.setEnabled(false);
					btnSave.setEnabled(false);
					mnuFileSave.setEnabled(false);
					btnOpen.setEnabled(false);
					mnuFileOpen.setEnabled(false);					      
					cmbZoomSelect.setEnabled(false);
				}
			}
			public void audioErrorOccurred(String s){
				messageBox(s);
			}
		});

		// set up the frame look

		setJMenuBar(mbrMenuBar);
        Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(tbrButtonBar, BorderLayout.NORTH);
		contentPane.add(pnlStatusBar, BorderLayout.SOUTH);
		contentPane.add(new JScrollPane(scopeDisplay));

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch (Exception e) {
			e.printStackTrace();
		}

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

	/********************************************************/
	/**
	 * Internal shortcut for displaying an error message box.
	 */
	private void messageBox(String s){
		JOptionPane.showMessageDialog(this, s, "Audio Capture Tool", JOptionPane.WARNING_MESSAGE);
	}

	/********************************************************/
	/**
	 * Internal shortcut for updating GUI strings.
	 */
	private void updateUIStrings(){
		if (player.hasAudio()){
	        setTitle("Audio Capture Tool - " + player.getAudioDescription());
	        txtAudioFormat.setText(player.getFormat().toString());
	    }else{
		    setTitle("Audio Capture Tool");
		    txtAudioFormat.setText(""); 
	    }
	}
	
	/********************************************************/
	/**
	 * Create and display and instance of the application.
	 */
    public static void main(String args[]) {
        JCapture c = new JCapture();
        c.show();
        c.setSize(450, 400);
	}
}
